package com.zjyr.beidouservice.adapter.impl.netty;

import com.zjyr.beidouservice.adapter.impl.netty.notify.SocketNotifyEvent;
import com.zjyr.beidouservice.common.impl.SocketEventName;
import com.zjyr.beidouservice.pojo.vo.binary.BaseBinaryProtocol;
import io.netty.channel.Channel;
import io.netty.channel.ChannelHandler;
import io.netty.channel.ChannelHandlerContext;
import io.netty.channel.SimpleChannelInboundHandler;
import io.netty.handler.timeout.IdleState;
import io.netty.handler.timeout.IdleStateEvent;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.ApplicationEventPublisher;
import org.springframework.stereotype.Component;

import java.net.InetSocketAddress;

@Slf4j
@Component
@ChannelHandler.Sharable
@RequiredArgsConstructor
public class MessageHandler<T> extends SimpleChannelInboundHandler<BaseBinaryProtocol<T>> {
    @Resource
    ApplicationEventPublisher applicationEventPublisher;

    @Override
    public void userEventTriggered(ChannelHandlerContext ctx, Object evt) throws Exception {
        if (evt instanceof IdleStateEvent idleStateEvent) {
            if (idleStateEvent.state() == IdleState.ALL_IDLE) {
                log.info("{}:: Trigger Heart Signe", ctx.channel().id());
                SocketNotifyEvent notify = new SocketNotifyEvent(this, ctx.channel(), SocketEventName.SOCKET_EVT_IDLE_TIMEOUT, null);
                applicationEventPublisher.publishEvent(notify);
            }
        } else {
            super.userEventTriggered(ctx, evt);
        }
    }

    @Override
    public void channelRead0(ChannelHandlerContext ctx, BaseBinaryProtocol<T> message) {
        SocketNotifyEvent notify = new SocketNotifyEvent(this,
                                                         ctx.channel(),
                                                         message.getTimeStamp(),
                                                         SocketEventName.SOCKET_EVT_RECV,
                                                         message.getMsgContent().getMsgBody());
        applicationEventPublisher.publishEvent(notify);
    }

    @Override
    public void channelActive(ChannelHandlerContext ctx) throws Exception {
        InetSocketAddress sa = (InetSocketAddress) ctx.channel().remoteAddress();
        log.info("{}:: Connected <-- {}:{}", ctx.channel().id(), sa.getAddress().getHostAddress(), sa.getPort());
        super.channelActive(ctx);
        SocketNotifyEvent notify = new SocketNotifyEvent(this, ctx.channel(), SocketEventName.SOCKET_EVT_CONNECT, null);
        applicationEventPublisher.publishEvent(notify);
    }

    @Override
    public void channelInactive(ChannelHandlerContext ctx) throws Exception {
        InetSocketAddress sa = (InetSocketAddress) ctx.channel().remoteAddress();
        log.info("{}:: Disonnected <-- {}", ctx.channel().id(), sa.getAddress().getHostAddress());

        super.channelInactive(ctx);
        ctx.close();

        SocketNotifyEvent notify = new SocketNotifyEvent(this, ctx.channel(), SocketEventName.SOCKET_EVT_DISCONNECT, null);
        applicationEventPublisher.publishEvent(notify);
    }

    public void channelSendData(Channel ctxChannel, Object proMsg) {
        if (ctxChannel != null && ctxChannel.isActive()) {
            ctxChannel.writeAndFlush(proMsg);
        }
    }
}
