package com.zjyr.beidouservice.adapter.impl.netty.decode;

import com.zjyr.beidouservice.common.CommonEnumHandler;
import com.zjyr.beidouservice.common.impl.ControlChannelStatus;
import com.zjyr.beidouservice.common.impl.ControlCommandName;
import com.zjyr.beidouservice.common.impl.SendStatusName;
import com.zjyr.beidouservice.common.impl.SensorControlTunnelName;
import com.zjyr.beidouservice.common.impl.TaskResultName;
import com.zjyr.beidouservice.misc.HelperUtils;
import com.zjyr.beidouservice.pojo.vo.binary.BaseBinaryProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.ControllerStatus;
import com.zjyr.beidouservice.pojo.vo.binary.MessageContent;
import com.zjyr.beidouservice.pojo.vo.binary.SensorStatusAck;
import com.zjyr.beidouservice.pojo.vo.binary.SensorStatusInfo;
import com.zjyr.beidouservice.pojo.vo.binary.SensorTaskAck;
import com.zjyr.beidouservice.pojo.vo.binary.SensorTaskStatus;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.ByteToMessageDecoder;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;

@Component
@RequiredArgsConstructor
@Slf4j
public class YuanRongProtocolDecode extends ByteToMessageDecoder {
    @Override
    protected void decode(ChannelHandlerContext channelHandlerContext, ByteBuf buf, List<Object> list) {
        log.info("\n{}", ByteBufUtil.prettyHexDump(buf));
        try {
            short msgLength  = buf.readShort();
            byte  version    = buf.readByte();
            byte  cryptCytp  = buf.readByte();
            int   timeStamp  = buf.readInt();
            int   statusCode = buf.readInt();
            byte  msgType    = buf.readByte();
            short msgSize    = buf.readShort();

            ControlCommandName cmd = CommonEnumHandler.codeOf(ControlCommandName.class, msgType);
            if (cmd != null) {
                switch (cmd) {
                    case COMMAND_REPORT_HEART -> {
                        int                  heartSeq       = buf.readInt();
                        SendStatusName       sendStatus     = CommonEnumHandler.codeOf(SendStatusName.class, buf.readByte());
                        ControlChannelStatus beidouStatus   = CommonEnumHandler.codeOf(ControlChannelStatus.class, buf.readByte());
                        byte[]               beidouCommFreq = new byte[3];
                        buf.readBytes(beidouCommFreq, 0, 3);
                        byte[] beidouSignal = new byte[10];
                        buf.readBytes(beidouSignal, 0, 10);
                        ControlChannelStatus wireStatus  = CommonEnumHandler.codeOf(ControlChannelStatus.class, buf.readByte());
                        ControlChannelStatus phoneStatus = CommonEnumHandler.codeOf(ControlChannelStatus.class, buf.readByte());

                        ControllerStatus status = ControllerStatus.builder()
                                                                  .heartSeq(heartSeq)
                                                                  .sendStatus(sendStatus)
                                                                  .beidouStatus(beidouStatus)
                                                                  .beidouFreq(beidouCommFreq)
                                                                  .beidouSignalStrength(beidouSignal)
                                                                  .wirelessStatus(wireStatus)
                                                                  .telphoneStatus(phoneStatus)
                                                                  .build();

                        MessageContent<ControllerStatus> msgCtx = MessageContent.<ControllerStatus>builder().msgType(msgType).msgSize(
                                msgSize).msgBody(status).build();

                        list.add(BaseBinaryProtocol.<ControllerStatus>builder()
                                                   .msgLength(msgLength)
                                                   .version(version)
                                                   .cryptoType(cryptCytp)
                                                   .timeStamp(timeStamp)
                                                   .statusCode(statusCode)
                                                   .msgContent(msgCtx)
                                                   .build());
                    }
                    case COMMAND_REPORT_SENSOR -> {
                        int  taskId = buf.readInt();
                        byte nItem  = buf.readByte();

                        SensorTaskAck taskAck = SensorTaskAck.builder().taskId(taskId).ackTimestamp(timeStamp).sensorStatus(new ArrayList<>(
                                nItem)).build();
                        for (int i = 0; i < nItem; i++) {
                            SensorTaskStatus taskStatus = SensorTaskStatus.builder()
                                                                          .sensorId(buf.readInt())
                                                                          .taskResult(CommonEnumHandler.codeOf(TaskResultName.class,
                                                                                                               buf.readByte()))
                                                                          .tunnelName(CommonEnumHandler.codeOf(SensorControlTunnelName.class,
                                                                                                               buf.readByte()))
                                                                          .build();

                            taskAck.getSensorStatus().add(taskStatus);
                        }

                        MessageContent<SensorTaskAck> msgCtx = MessageContent.<SensorTaskAck>builder()
                                                                             .msgType(msgType)
                                                                             .msgSize(msgSize)
                                                                             .msgBody(taskAck)
                                                                             .build();

                        list.add(BaseBinaryProtocol.<SensorTaskAck>builder()
                                                   .msgLength(msgLength)
                                                   .version(version)
                                                   .cryptoType(cryptCytp)
                                                   .timeStamp(timeStamp)
                                                   .statusCode(statusCode)
                                                   .msgContent(msgCtx)
                                                   .build());
                    }
                    case COMMAND_REPORT_QUERY_SENSOR -> {
                        int             nItems    = buf.readByte();
                        SensorStatusAck sensorAck = SensorStatusAck.builder().sensorStatus(new ArrayList<>()).build();

                        MessageContent<SensorStatusAck> msgCtx = MessageContent.<SensorStatusAck>builder()
                                                                               .msgType(msgType)
                                                                               .msgSize(msgSize)
                                                                               .msgBody(sensorAck)
                                                                               .build();

                        for (int i = 0; i < nItems; i++) {
                            short                   tmInfo     = buf.readShort();
                            SensorControlTunnelName tn         = CommonEnumHandler.codeOf(SensorControlTunnelName.class, buf.readByte());
                            int                     cmdType    = buf.readByte();
                            short                   sa         = buf.readShort();
                            short                   sid        = buf.readShort();
                            int                     warrStatus = buf.readByte();
                            int                     emg0       = buf.readByte();
                            int                     emg1       = buf.readByte();
                            int                     emg2       = buf.readByte();
                            int                     spk0       = buf.readByte();
                            int                     spk1       = buf.readByte();
                            int                     spk2       = buf.readByte();
                            int                     runStatus  = buf.readByte();
                            byte[]                  signale    = new byte[5];
                            buf.readBytes(signale, 0, 5);
                            byte[] id = new byte[3];
                            buf.readBytes(id, 0, 3);

                            SensorStatusInfo si = SensorStatusInfo.builder()
                                                                  .dtInfo(tmInfo)
                                                                  .tunnelName(tn)
                                                                  .command(cmdType)
                                                                  .startAddr(sa)
                                                                  .sensorId(sid)
                                                                  .warrningStatus(warrStatus)
                                                                  .emergenStatus0(emg0)
                                                                  .emergenStatus1(emg1)
                                                                  .emergenStatus2(emg2)
                                                                  .speakerStatus0(spk0)
                                                                  .speakerStatus1(spk1)
                                                                  .speakerStatus2(spk2)
                                                                  .runStatus(runStatus)
                                                                  .signalStrength(HelperUtils.bytesToHexString(signale))
                                                                  .beidouId(HelperUtils.bytesToHexString(id))
                                                                  .build();

                            sensorAck.getSensorStatus().add(si);
                        }

                        list.add(BaseBinaryProtocol.<SensorStatusAck>builder()
                                                   .msgLength(msgLength)
                                                   .version(version)
                                                   .cryptoType(cryptCytp)
                                                   .timeStamp(timeStamp)
                                                   .statusCode(statusCode)
                                                   .msgContent(msgCtx)
                                                   .build());
                    }
                    default -> log.error("Unsupport Command: {}({})", cmd, msgType);
                }
            }
        } catch (Exception ex) {
            log.error("Clear buffer because error data");
            buf.resetReaderIndex();
            buf.resetWriterIndex();
        }

    }
}
