package com.zjyr.beidouservice.adapter.impl.netty.encode;

import com.zjyr.beidouservice.common.impl.ControlCommandName;
import com.zjyr.beidouservice.pojo.vo.binary.BaseBinaryProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.HeartProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.QuerySensorProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.SensorControlProtocol;
import io.netty.buffer.ByteBuf;
import io.netty.buffer.ByteBufUtil;
import io.netty.channel.ChannelHandlerContext;
import io.netty.handler.codec.MessageToByteEncoder;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

@Component
@RequiredArgsConstructor
@Slf4j
public class YuanRongProtocolEncode<T> extends MessageToByteEncoder<BaseBinaryProtocol<T>> {
    @Override
    protected void encode(ChannelHandlerContext channelHandlerContext, BaseBinaryProtocol<T> baseBinaryProtocol, ByteBuf byteBuf) {

        if (baseBinaryProtocol == null || baseBinaryProtocol.getMsgContent() == null ||
            baseBinaryProtocol.getMsgContent().getMsgBody() == null) {
            return;
        }

        T msgBody = baseBinaryProtocol.getMsgContent().getMsgBody();

        byteBuf.writeByte(BaseBinaryProtocol.VERSIN & 0xFF);       // 协议版本号
        byteBuf.writeByte(baseBinaryProtocol.getCryptoType());       // 协议加密选项
        byteBuf.writeInt((int) (System.currentTimeMillis() / 1000)); // UTC时间戳，单位秒，协议发送时的UTC时间
        byteBuf.writeInt(baseBinaryProtocol.getStatusCode());        // 消息状态码

        if (msgBody instanceof HeartProtocol heartProtocol) {
            byteBuf.writeByte(ControlCommandName.COMMAND_HEART.getValue().byteValue());    // 消息类型
            byteBuf.writeShort(4);      // 消息内容长度
            byteBuf.writeInt(heartProtocol.getSeqId());
        } else if (msgBody instanceof SensorControlProtocol msg) {
            byteBuf.writeByte(ControlCommandName.COMMAND_SENSOR.getValue().byteValue());    // 消息类型
            // 消息内容长度
            byteBuf.writeShort(10 + 3 + msg.getCityCode().size() + msg.getDistrictsCode().size() + msg.getSensorId().size() * 4);
            byteBuf.writeInt(msg.getTaskId());
            byteBuf.writeByte(msg.getAlarmMode().getValue().byteValue());
            byteBuf.writeByte(msg.getAlarmMode().getValue().byteValue());
            byteBuf.writeByte(msg.getAlarmType().getValue().byteValue());
            byteBuf.writeByte(msg.getControlTunnel().byteValue());
            byteBuf.writeInt(msg.getTimeStamp());

            if (msg.getCityCode().isEmpty()) {
                byteBuf.writeByte(0);
            } else {
                byteBuf.writeByte(msg.getCityCode().size());
                for (var c : msg.getCityCode()) {
                    byteBuf.writeByte(c.byteValue());
                }
            }

            if (msg.getDistrictsCode().isEmpty()) {
                byteBuf.writeByte(0);
            } else {
                byteBuf.writeByte(msg.getDistrictsCode().size());
                for (var c : msg.getDistrictsCode()) {
                    byteBuf.writeByte(c.byteValue());
                }
            }

            if (msg.getSensorId().isEmpty()) {
                byteBuf.writeByte(0);
            } else {
                byteBuf.writeByte(msg.getSensorId().size());
                for (var c : msg.getSensorId()) {
                    byteBuf.writeInt(c);
                }
            }
        } else if (msgBody instanceof QuerySensorProtocol msg) {
            byteBuf.writeByte(ControlCommandName.COMMAND_QUERY_SENSOR.getValue().byteValue());    // 消息类型

//            if (!msg.getControllContents().isEmpty()) {
//                byteBuf.writeShort(2 + 4 * msg.getControllContents().size());      // 消息内容长度
//                byteBuf.writeByte(msg.getProvince());
//                byteBuf.writeByte(msg.getCity());
//                byteBuf.writeByte(msg.getNControlInfo());
//                for (var c : msg.getControllContents()) {
//                    byteBuf.writeByte(c.getDistrictsCode());
//                    //byteBuf.writeByte(c.getControlAction().getValue().byteValue());
//                    byteBuf.writeShort(c.getSensorId());
//                }
//            }
        } else {
            log.info("XXXXX Encode: {}", baseBinaryProtocol.getMsgContent().getMsgBody().getClass());
        }

        log.info("\n{}", ByteBufUtil.prettyHexDump(byteBuf));
    }
}
