package com.zjyr.beidouservice.adapter.impl.netty.impl;

import com.zjyr.beidouservice.adapter.impl.netty.ChannelInit;
import com.zjyr.beidouservice.adapter.impl.netty.ISocketServer;
import com.zjyr.beidouservice.config.NettySocketConfigure;
import io.netty.bootstrap.ServerBootstrap;
import io.netty.channel.Channel;
import io.netty.channel.ChannelOption;
import io.netty.channel.EventLoopGroup;
import io.netty.channel.nio.NioEventLoopGroup;
import io.netty.channel.socket.SocketChannel;
import io.netty.channel.socket.nio.NioServerSocketChannel;
import jakarta.annotation.PreDestroy;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Component;

import java.net.InetSocketAddress;

@Component
@Slf4j
@RequiredArgsConstructor
public class TcpServer implements ISocketServer {
    @Resource
    private final ChannelInit<SocketChannel> channelInit;

    @Resource
    private final NettySocketConfigure nettySocketConfigure;
    private       EventLoopGroup       boosGroup;
    private       EventLoopGroup       workerGroup;

    private void tcpServer(int port) {
        try {
            new ServerBootstrap().group(boosGroup, workerGroup)
                                 .channel(NioServerSocketChannel.class)
                                 .localAddress(new InetSocketAddress(port))
                                 .childHandler(channelInit)
                                 .option(ChannelOption.SO_BACKLOG, 128)
                                 .childOption(ChannelOption.TCP_NODELAY, false)
                                 .childOption(ChannelOption.SO_KEEPALIVE, true)
                                 .bind()
                                 .sync();
            log.info("Netty TCP Server Beginning Listen: {}", port);
        } catch (Exception e) {
            log.error(e.getMessage());
            boosGroup.shutdownGracefully();
            workerGroup.shutdownGracefully();
        }
    }

    @Override
    public void start() {
        log.info("Start Netty TCP Server");
        boosGroup   = new NioEventLoopGroup();
        workerGroup = new NioEventLoopGroup();
        this.tcpServer(nettySocketConfigure.getServerPort());
    }

    @PreDestroy
    @Override
    public void destory() {

    }

    @Override
    public void sendData(Channel ch, Object data) {
        channelInit.getMessageHandler().channelSendData(ch, data);
    }
}
