package com.zjyr.beidouservice.service.impl;

import com.zjyr.beidouservice.common.impl.AlarmControlTypeName;
import com.zjyr.beidouservice.common.impl.AlarmModeName;
import com.zjyr.beidouservice.common.impl.AlarmTypeName;
import com.zjyr.beidouservice.pojo.vo.binary.BaseBinaryProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.HeartProtocol;
import com.zjyr.beidouservice.pojo.vo.binary.MessageContent;
import com.zjyr.beidouservice.pojo.vo.binary.SensorControlProtocol;
import com.zjyr.beidouservice.service.AdapterProtocolService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.sql.Timestamp;
import java.util.Date;
import java.util.List;

@Service
@Slf4j
public class BeidouSocketProtocolServiceImpl implements AdapterProtocolService {
    private int currentSeqId = 0;

    @Override
    public HeartProtocol createHeartProtocol() {
        return new HeartProtocol(currentSeqId++);
    }

    @Override
    public SensorControlProtocol createSensorControlProtocol(int taskId,
                                                             AlarmModeName alarmMode,
                                                             AlarmControlTypeName controlType,
                                                             AlarmTypeName alarmType,
                                                             int tunnelName,
                                                             Date alarmDt,
                                                             List<Integer> citys,
                                                             List<Integer> areas,
                                                             List<Integer> sensors) {
        long timestamp = new Timestamp(alarmDt.getTime()).getTime() / 1000;
        return SensorControlProtocol.builder()
                                    .taskId(taskId)
                                    .alarmMode(alarmMode)
                                    .alarmControlType(controlType)
                                    .alarmType(alarmType)
                                    .controlTunnel(tunnelName)
                                    .timeStamp((int) timestamp)
                                    .cityCode(citys)
                                    .districtsCode(areas)
                                    .sensorId(sensors)
                                    .build();
    }

//
//    @Override
//    public List<SensorControllContent> createSensorCtrlInfo(List<BeidouAdapterControlContent> ctxList) {
//        List<SensorControllContent> list = new ArrayList<>();
//
//        for (BeidouAdapterControlContent v : ctxList) {
//            list.add(SensorControllContent.builder()
//                                          .districtsCode(v.getDistrictsCode().byteValue())
//                                          //.controlAction(v.getControlAction())
//                                          .sensorId(v.getSensorId().shortValue())
//                                          .build());
//        }
//
//        return list;
//    }
//
//    @Override
//    public SensorControlProtocol createSensorControlProtocol(SensorControlTunnelName tunnelName,
//                                                             byte[] tunnelAddr,
//                                                             Integer month,
//                                                             Integer days,
//                                                             Integer hour,
//                                                             Integer minute,
//                                                             Integer province,
//                                                             Integer city,
//                                                             SensorControlModeName ctrlMode,
//                                                             Integer ctrlCmd,
//                                                             List<SensorControllContent> ctrlItems) {
//        return SensorControlProtocol.builder()
//                                    .controlTunnel(tunnelName)
//                                    .tunnelAddr(tunnelAddr)
//                                    .month(month.byteValue())
//                                    .days(days.byteValue())
//                                    .hour(hour.byteValue())
//                                    .minute(minute.byteValue())
//                                    .province(province.byteValue())
//                                    .city(city.byteValue())
//                                    .controlMode(ctrlMode)
//                                    .controlCommand(ctrlCmd.byteValue())
//                                    .nControlInfo(((Integer) ctrlItems.size()).byteValue())
//                                    .controllContents(new ArrayList<>(ctrlItems))
//                                    .build();
//    }
//
//    @Override
//    public QuerySensorProtocol createSensorQueryProtocol(Integer province,
//                                                         Integer city,
//                                                         List<SensorControllContent> ctrlItems) {
//        return QuerySensorProtocol.builder()
//                                  .province(province.byteValue())
//                                  .city(city.byteValue())
//                                  .nControlInfo(((Integer) ctrlItems.size()).byteValue())
//                                  .controllContents(new ArrayList<>(ctrlItems))
//                                  .build();
//    }

    @Override
    public <T> Object createTransmissionProtocol(T proContent) {
        MessageContent<T> msgCont = MessageContent.<T>builder().msgBody(proContent).build();
        return BaseBinaryProtocol.<T>builder().msgContent(msgCont).build();
    }
}
