package com.zjyr.beidouservice.service.impl;

import com.zjyr.beidouservice.constenum.AlarmTypeEnum;
import com.zjyr.beidouservice.constenum.ManufacturerTypeEnum;
import com.zjyr.beidouservice.mapper.DevAttributesMapper;
import com.zjyr.beidouservice.mapper.DevBasicInfoMapper;
import com.zjyr.beidouservice.pojo.dataobject.DevAttributesDO;
import com.zjyr.beidouservice.pojo.dataobject.DevBasicInfoDO;
import com.zjyr.beidouservice.pojo.dto.AlarmDevContentDTO;
import com.zjyr.beidouservice.pojo.dto.AlarmDevRespDTO;
import com.zjyr.beidouservice.pojo.dto.DevMaunDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceInfoByLocDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceReqDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceRespDTO;
import com.zjyr.beidouservice.pojo.dto.ManufacturerContentDTO;
import com.zjyr.beidouservice.pojo.dto.ManufacturerRespDTO;
import com.zjyr.beidouservice.service.DeviceService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;
import java.util.stream.Collectors;

@Service
@Slf4j
public class DeviceServiceImpl implements DeviceService {

    @Resource
    DevBasicInfoMapper devBasicInfoMapper;

    @Resource
    DevAttributesMapper devAttributesMapper;

    @Override
    public List<DeviceRespDTO> getDeviceInfo(DeviceReqDTO deviceReqDTO) {
        log.info("get device info : [{}]", deviceReqDTO);
        List<DeviceRespDTO> deviceRespList = new ArrayList<>();

        //根据省份、地市、区信息查询设备
        DeviceInfoByLocDTO searchDTO = new DeviceInfoByLocDTO();
        searchDTO.setProvinceCode(deviceReqDTO.getProvinceCode());
        searchDTO.setCityCode(deviceReqDTO.getCityCode());
        searchDTO.setCountyCode(deviceReqDTO.getCountyCode());
        searchDTO.setDeviceName(deviceReqDTO.getDeviceName());
        log.info("device search info : [{}]", searchDTO);

        List<DevBasicInfoDO> devBasicInfoList = devBasicInfoMapper.selectDeviceByLoc(searchDTO);
        if (devBasicInfoList.isEmpty()) {
            log.warn("根据省份、地市、区编码未查找到设备信息");
            return new ArrayList<>();
        }

        devBasicInfoList.forEach(dev -> {
            Long             deviceId     = dev.getDeviceId();
            String           manufacturer = deviceReqDTO.getManufacturer();
            List<DevMaunDTO> devMaunDTO   = devAttributesMapper.selectManuByManufacturer(deviceId, manufacturer);
            //根据过滤过厂商的device查找device基本信息
            devMaunDTO.forEach(v -> {
                DeviceRespDTO deviceResp = new DeviceRespDTO();
                deviceResp.setDeviceId(v.getDeviceId());
                deviceResp.setAlarmManufacturer(ManufacturerTypeEnum.getByCode(v.getAlarmManufacturer()));
                deviceResp.setControlManufacturer(ManufacturerTypeEnum.getByCode(v.getControlManufacturer()));

                //根据deviceId查找device basic info
                DevBasicInfoDO devBasicInfo = devBasicInfoMapper.selectDeviceByDeviceId(v.getDeviceId());
                deviceResp.setDeviceName(devBasicInfo.getDeviceName());
                deviceResp.setProvinceCode(devBasicInfo.getProvinceCode());
                deviceResp.setCityCode(devBasicInfo.getCityCode());
                deviceResp.setCountyCode(devBasicInfo.getCountyCode());

                //获取设备安装地址、经纬度
                deviceResp.setInstallAddr(devBasicInfo.getInstallAddr());
                deviceResp.setLatitude(devBasicInfo.getLatitude());
                deviceResp.setLatitude(devBasicInfo.getLatitude());

                deviceRespList.add(deviceResp);
            });
        });

        return deviceRespList;
    }

    @Override
    public ManufacturerRespDTO getDeviceManufacturerInfo() {
        ManufacturerRespDTO manufacturerResp = new ManufacturerRespDTO();
        //获取厂家名称和类型
        List<ManufacturerContentDTO> contents = new ArrayList<>();
        List<Integer>                codeList = ManufacturerTypeEnum.getAllCodes();
        codeList.forEach(v -> {
            ManufacturerContentDTO content = new ManufacturerContentDTO();
            content.setManufacturerType(v);
            content.setManufacturerName(ManufacturerTypeEnum.getByCode(v));

            contents.add(content);
        });

        manufacturerResp.setData(contents);
        return manufacturerResp;
    }

    @Override
    public AlarmDevRespDTO getAlarmDeviceInfo() {
        AlarmDevRespDTO          respDTO  = new AlarmDevRespDTO();
        List<AlarmDevContentDTO> contents = new ArrayList<>();

        //获取所有的设备属性信息
        List<DevAttributesDO> devMaunDTOList = devAttributesMapper.selectManuAll();
        if (devMaunDTOList.isEmpty()) {
            log.warn("不存在告警终端");
            return new AlarmDevRespDTO();
        }

        int totalMum = devMaunDTOList.size();

        //分别过滤出不同的警报器类型
        List<Integer> typeList = AlarmTypeEnum.getAllCodes();
        typeList.forEach(type -> {
            AlarmDevContentDTO content = new AlarmDevContentDTO();
            String             typeMsg = AlarmTypeEnum.getByCode(type);
            log.info("alarm type :[{}]", typeMsg);

            //过滤出不同类型的设备属性
            List<DevAttributesDO> devType = devMaunDTOList.stream().filter(v -> v.getAlarmType().equals(type)).collect(Collectors.toList());
            content.setAlarmType(typeMsg);
            content.setAlarmNum(devType.size());
            double ratio = (double) devType.size() / (double) totalMum;
            content.setAlarmRatio(ratio);

            contents.add(content);
        });

        respDTO.setAlarmDevTotalNum(totalMum);
        respDTO.setData(contents);
        return respDTO;
    }
}
