package com.zjyr.beidouservice.service.impl;

import com.zjyr.beidouservice.mapper.DevTaskMapper;
import com.zjyr.beidouservice.mapper.TaskMapper;
import com.zjyr.beidouservice.pojo.dataobject.DevTaskDO;
import com.zjyr.beidouservice.pojo.dataobject.TaskDO;
import com.zjyr.beidouservice.pojo.dto.DevTaskExecRespDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceTaskContentDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceTaskExecReqDTO;
import com.zjyr.beidouservice.pojo.dto.DeviceTaskRespDTO;
import com.zjyr.beidouservice.service.DeviceTaskService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;

@Service
@Slf4j
public class DeviceTaskServiceImpl implements DeviceTaskService {

    @Resource
    DevTaskMapper devTaskMapper;

    @Resource
    TaskMapper taskMapper;

    @Override
    public List<DeviceTaskRespDTO> getDeviceTaskInfo() {
        log.info(" get all device task information");
        List<DeviceTaskRespDTO> respDTOList = new ArrayList<>();

        //获取所有任务id
        List<TaskDO> taskDOList = taskMapper.selectAllTask();
        if (taskDOList.isEmpty()) {
            log.warn("不存在终端设备告警任务");
            return new ArrayList<>();
        }

        taskDOList.forEach(task -> {
            DeviceTaskRespDTO taskResp = new DeviceTaskRespDTO();
            taskResp.setId(task.getId());
            taskResp.setTaskId(task.getTaskId());
            //告警任务记录时间为任务创建时间
            taskResp.setRecodeTime(task.getCreateTime());


            //告警任务回示率,收到响应信息的设备数量(执行成功、执行失败)/总设备数量
            int    respNum   = task.getExecNormalNum() + task.getExecAbnormalNum();
            int    deviceNum = task.getDeviceNum();
            double respRate  = (double) respNum / (double) deviceNum;

            //告警任务执行设备数量、执行正常数量、执行异常数量、回示率
            taskResp.setDeviceNum(deviceNum);
            taskResp.setExecNormalNum(task.getExecNormalNum());
            taskResp.setExecAbnormalNum(task.getExecAbnormalNum());
            taskResp.setRespRate(respRate);

            //获取任务id详细信息
            List<DeviceTaskContentDTO> devTaskContents = new ArrayList<>();
            List<DevTaskDO>            devTaskDOList   = devTaskMapper.selectDevTaskByTaskId(task.getTaskId());
            if (devTaskDOList.isEmpty()) {
                log.warn("根据任务Id: [{}] 获取不到告警任务详情信息", task.getTaskId());
                taskResp.setData(new ArrayList<>());
            }

            devTaskDOList.forEach(devTask -> {
                DeviceTaskContentDTO devTaskContent = new DeviceTaskContentDTO();
                devTaskContent.setDeviceId(devTask.getDeviceId());
                devTaskContent.setDeviceName(devTask.getDeviceName());
                //回示时间为dev task表中的更新时间，收到执行状态后更新时间
                devTaskContent.setRespTime(devTask.getUpdateTime());
                devTaskContent.setExecState(devTask.getExecState());

                devTaskContents.add(devTaskContent);

            });

            taskResp.setData(devTaskContents);
            respDTOList.add(taskResp);

        });

        // 使用自定义比较器按recodeTime排序
        respDTOList.sort(new Comparator<DeviceTaskRespDTO>() {
            public int compare(DeviceTaskRespDTO t1, DeviceTaskRespDTO t2) {
                return t2.getRecodeTime().compareTo(t1.getRecodeTime());
            }
        });

        return respDTOList;
    }


    @Override
    public DevTaskExecRespDTO deviceTaskExec(DeviceTaskExecReqDTO deviceTaskExecReqDTO) {
        DevTaskExecRespDTO deviceTaskRespDTO = new DevTaskExecRespDTO();

        //todo  北斗号 beidouId，如果以市为单位，则携带市北斗号，区为单位携带区北斗号，具体设备则携带区北斗号

        //todo  控制类型 controlType:。1:单点控制 2:市属控制 3:区属控制

        return deviceTaskRespDTO;
    }
}
