package com.zjyr.beidouservice.service.impl;

import com.zjyr.beidouservice.mapper.LocCityMapper;
import com.zjyr.beidouservice.mapper.LocCountyMapper;
import com.zjyr.beidouservice.mapper.LocProvinceMapper;
import com.zjyr.beidouservice.pojo.dataobject.LocCityDO;
import com.zjyr.beidouservice.pojo.dataobject.LocCountyDO;
import com.zjyr.beidouservice.pojo.dataobject.LocProvinceDO;
import com.zjyr.beidouservice.pojo.dto.city.CityCountyDTO;
import com.zjyr.beidouservice.pojo.dto.city.CityRespDTO;
import com.zjyr.beidouservice.pojo.dto.city.CitySearchDTO;
import com.zjyr.beidouservice.pojo.dto.city.CountyContent;
import com.zjyr.beidouservice.pojo.dto.province.CityContentDTO;
import com.zjyr.beidouservice.pojo.dto.province.OnlyProvinceContentDTO;
import com.zjyr.beidouservice.pojo.dto.province.OnlyProvinceResp;
import com.zjyr.beidouservice.pojo.dto.province.ProvinceContent;
import com.zjyr.beidouservice.pojo.dto.province.ProvinceRespDTO;
import com.zjyr.beidouservice.pojo.dto.province.ProvinceSearchDTO;
import com.zjyr.beidouservice.service.LocationService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class LocationServiceImpl implements LocationService {
    @Resource
    LocProvinceMapper locProvinceMapper;

    @Resource
    LocCityMapper locCityMapper;

    @Resource
    LocCountyMapper locCountyMapper;

    @Override
    public OnlyProvinceResp getOnlyProvinces() {
        OnlyProvinceResp             onlyProvinceResp = new OnlyProvinceResp();
        List<OnlyProvinceContentDTO> onlyProvinceList = new ArrayList<>();

        //获取省份信息
        List<LocProvinceDO> provinceDOS = locProvinceMapper.selectAllProvince();
        if (provinceDOS.isEmpty()) {
            log.warn("不存在省份信息");
            return new OnlyProvinceResp();
        }

        provinceDOS.forEach(province -> {
            OnlyProvinceContentDTO onlyProvinceContent = new OnlyProvinceContentDTO();
            onlyProvinceContent.setProvinceName(province.getProvinceName());
            onlyProvinceContent.setProvinceCode(province.getProvinceCode());
            onlyProvinceList.add(onlyProvinceContent);
        });

        onlyProvinceResp.setOnlyProvinceList(onlyProvinceList);
        return onlyProvinceResp;
    }

    @Override
    public ProvinceRespDTO getProvinces(ProvinceSearchDTO provinceSearchDTO) {
        ProvinceRespDTO       provinceResp     = new ProvinceRespDTO();
        List<ProvinceContent> provinceContents = new ArrayList<>();

        //获取省份信息
        String              provinceCode = provinceSearchDTO.getProvinceCode();
        List<LocProvinceDO> provinceDOS  = locProvinceMapper.selectProvince(provinceCode);
        if (provinceDOS.isEmpty()) {
            log.warn("不存在省份信息");
            return new ProvinceRespDTO();
        }


        provinceDOS.forEach(province -> {
            ProvinceContent provinceContent = new ProvinceContent();
            provinceContent.setProvinceName(province.getProvinceName());
            provinceContent.setProvinceCode(province.getProvinceCode());

            //根据省份查询对应的地市信息
            List<CityContentDTO> cityContentDTOList = new ArrayList<>();
            List<LocCityDO>      cityDOList         = locCityMapper.selectCityByProvince(province.getProvinceCode());
            cityDOList.forEach(city -> {
                CityContentDTO cityContent = new CityContentDTO();
                cityContent.setCityName(city.getCityName());
                cityContent.setCityCode(city.getCityCode());
                cityContentDTOList.add(cityContent);
            });

            provinceContent.setCityList(cityContentDTOList);
            provinceContents.add(provinceContent);

        });

        provinceResp.setProvinceList(provinceContents);
        return provinceResp;
    }


    @Override
    public CityRespDTO getCity(CitySearchDTO citySearchDTO) {
        CityRespDTO         cityResp     = new CityRespDTO();
        List<CityCountyDTO> cityContents = new ArrayList<>();

        //获取地市信息
        String          provinceCode = citySearchDTO.getProvinceCode();
        String          cityCode     = citySearchDTO.getCityCode();
        List<LocCityDO> cityDOS      = locCityMapper.selectCity(provinceCode, cityCode);
        if (cityDOS.isEmpty()) {
            log.warn("不存在地市信息");
            return new CityRespDTO();
        }

        cityDOS.forEach(city -> {
            CityCountyDTO cityContent = new CityCountyDTO();
            cityContent.setProvinceCode(city.getProvinceCode());
            cityContent.setCityName(city.getCityName());
            cityContent.setCityCode(city.getCityCode());

            //根据地市查询对应的区信息
            List<CountyContent> countyContentList = new ArrayList<>();
            List<LocCountyDO>   countyDOList      = locCountyMapper.selectCountyByCity(city.getProvinceCode(), city.getCityCode());
            countyDOList.forEach(county -> {
                CountyContent countyContent = new CountyContent();
                countyContent.setCountyName(county.getCountyName());
                countyContent.setCountyCode(county.getCountyCode());
                countyContentList.add(countyContent);
            });

            cityContent.setCountyList(countyContentList);
            cityContents.add(cityContent);

        });

        cityResp.setCityList(cityContents);
        return cityResp;
    }

}
