package com.zjyr.beidouservice.service.impl;

import com.zjyr.beidouservice.mapper.SensorDataMapper;
import com.zjyr.beidouservice.pojo.entry.SensorData;
import com.zjyr.beidouservice.pojo.vo.binary.SensorStatusInfo;
import com.zjyr.beidouservice.service.SensorDataService;
import jakarta.annotation.Resource;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Service
@Slf4j
public class SensorDataServiceImpl implements SensorDataService {

    @Resource
    private SensorDataMapper sensorDataMapper;

    public static SensorStatusInfo createSensorInfoFromSensorData(SensorData s) {
        return SensorStatusInfo.builder()
                               .dtInfo(s.getDtInfo().shortValue())
                               .tunnelName(s.getChannelType())
                               .command(s.getCommand())
                               .startAddr(s.getStartAddr().shortValue())
                               .sensorId(s.getDeviceId().shortValue())
                               .warrningStatus(s.getWarrningStatus())
                               .emergenStatus0((s.getEmergStatus() >> 16) & 0xFF)
                               .emergenStatus1((s.getEmergStatus() >> 8) & 0xFF)
                               .emergenStatus2(s.getEmergStatus() & 0xFF)
                               .speakerStatus0((s.getSpearkStatus() >> 16) & 0xFF)
                               .speakerStatus1((s.getSpearkStatus() >> 8) & 0xFF)
                               .speakerStatus2(s.getSpearkStatus() & 0xFF)
                               .runStatus(s.getRunStatus())
                               .signalStrength(s.getSignalStrength())
                               .beidouId(s.getBeidouId())
                               .build();
    }

    private Integer createStatusMapValue(Integer val0, Integer val1, Integer val2) {
        return (val0 & 0xFF) << 16 | (val1 & 0xFF) << 8 | (val2 & 0xFF);
    }

    @Override
    public List<SensorStatusInfo> getAllSensor() {
        List<SensorStatusInfo> sList      = new ArrayList<>();
        List<SensorData>       sensorData = sensorDataMapper.selectAll();

        if (sensorData.isEmpty()) {
            return sList;
        }

        for (SensorData s : sensorData) {
            sList.add(createSensorInfoFromSensorData(s));
        }

        return sList;
    }

    @Override
    public void addNewSensor(List<SensorStatusInfo> sensorList) {
        for (SensorStatusInfo v : sensorList) {
            addNewSensor(v);
        }
    }

    @Override
    public void addNewSensor(SensorStatusInfo v) {
        SensorData s = SensorData.builder()
                                 .deviceId(v.getSensorId().intValue())
                                 .channelType(v.getTunnelName())
                                 .command(v.getCommand())
                                 .startAddr(v.getStartAddr().intValue())
                                 .dtInfo(v.getDtInfo().intValue())
                                 .warrningStatus(v.getWarrningStatus())
                                 .emergStatus(createStatusMapValue(v.getEmergenStatus0(), v.getEmergenStatus1(), v.getEmergenStatus2()))
                                 .spearkStatus(createStatusMapValue(v.getSpeakerStatus0(), v.getSpeakerStatus1(), v.getSpeakerStatus2()))
                                 .runStatus(v.getRunStatus())
                                 .signalStrength(v.getSignalStrength())
                                 .beidouId(v.getBeidouId())
                                 .build();

        SensorData sensorData = sensorDataMapper.getSensorDataById(s.getDeviceId());

        if (sensorData == null) {
            sensorDataMapper.addSensorData(s);
        } else {
            sensorDataMapper.upgradeSensorData(s);
        }
    }

    @Override
    public SensorStatusInfo getSensorDataById(Integer sensorId) {
        SensorData s = sensorDataMapper.getSensorDataById(sensorId);

        if (s != null) {
            return createSensorInfoFromSensorData(s);
        }

        return null;
    }
}
